from random import choice
from wtforms.widgets import HiddenInput
from wtforms.widgets.core import HTMLString


TAGS = ('strong', 'em', 'ol', 'ul')


class QuillInput(HiddenInput):
    """
    Renders the text content as hidden input and adds a container for the
    editor.


    """

    def __init__(self, **kwargs):
        tags = list(set(kwargs.pop('tags', TAGS)) & set(TAGS))

        super(QuillInput, self).__init__(**kwargs)

        id = ''.join(choice('abcdefghi') for i in range(8))
        self.container_id = 'quill-container-{}'.format(id)
        self.input_id = 'quill-input-{}'.format(id)

        self.formats = []
        if 'strong' in tags:
            self.formats.append("'bold'")
        if 'em' in tags:
            self.formats.append("'italic'")
        if 'ol' in tags or 'ul' in tags:
            self.formats.append("'list'")

        self.toolbar = []
        if 'strong' in tags:
            self.toolbar.append("'bold'")
        if 'em' in tags:
            self.toolbar.append("'italic'")
        if 'ol' in tags:
            self.toolbar.append("{list: 'ordered'}")
        if 'ul' in tags:
            self.toolbar.append("{list: 'bullet'}")

    def __call__(self, field, **kwargs):
        kwargs['id'] = self.input_id
        input = super(QuillInput, self).__call__(field, **kwargs)

        return HTMLString("""
                <div class="quill-container" id="{container_id}"></div>
                <script>
                    window.addEventListener('load', function () {{
                        var input = document.getElementById('{input_id}');
                        var quill = new Quill('#{container_id}', {{
                            formats: [{formats}],
                            modules: {{toolbar: [{toolbar}]}},
                            theme: 'snow'
                        }});
                        quill.clipboard.dangerouslyPasteHTML(input.value);
                        quill.on('text-change', function() {{
                            input.value = quill.root.innerHTML
                        }});
                    }});
                </script>
                {input}
        """.format(
            container_id=self.container_id,
            input_id=self.input_id,
            formats=', '.join(self.formats),
            toolbar=', '.join(self.toolbar),
            input=input
        ))



from bleach.sanitizer import Cleaner
from onegov.quill.widgets import QuillInput
from onegov.quill.widgets import TAGS
from wtforms import TextAreaField


class QuillField(TextAreaField):
    """ A textfield using the quill editor and with integrated sanitation.

    Allows to specifiy which tags to use in the editor and for sanitation.
    Available tags are: strong, em, ol and ul (p and br tags are always
    possible).

    """

    def __init__(self, **kwargs):
        tags = list(set(kwargs.pop('tags', TAGS)) & set(TAGS))
        super(TextAreaField, self).__init__(**kwargs)

        self.widget = QuillInput(tags=tags)

        tags = ['p', 'br'] + tags
        if 'ol' in tags or 'ul' in tags:
            tags.append('li')
        self.cleaner = Cleaner(tags=tags, attributes={}, strip=True)

    def pre_validate(self, form):
        self.data = self.cleaner.clean(self.data or '')



import morepath

from onegov.core import Framework
from onegov.core import filters  # noqa -> registers webasset filters
from onegov.core.utils import scan_morepath_modules
from onegov.quill import QuillApp
from webtest import TestApp as Client


def test_integration(temporary_directory):

    class App(Framework, QuillApp):
        pass

    @App.webasset_output()
    def get_output_path():
        return temporary_directory

    @App.path(path='')
    class Root(object):
        pass

    @App.html(model=Root)
    def view_root(self, request):
        request.include('quill')
        return '<body></body>'

    scan_morepath_modules(App)
    morepath.commit(App)

    app = App()
    app.configure_application()
    app.namespace = 'foo'
    app.set_application_id('foo/bar')

    assert 'quill.js.bundle.js' in Client(app).get('/')
